/**
 * Tidur för verifiering av WCAG-krav med tidsgränser.
 *
 * Skriptet skapar en knapp i varje element med class="timer". När användaren
 * trycker på knappen startar ett tidur med det antal sekunder som anges av
 * elementets data-duration-attribut.
 *
 * Om användaren stopar tiduret genom att trycka på knappen igen innan
 * aktiveringstiden löpt ut presenterar skriptet hur lång tid tiduret varit
 * igång.
 *
 * Om användaren inte stoppar tiduret presenterar skriptet den upplupna
 * tiden när tiden är ute.
 */
((win, doc) => {
    "use strict";


    /**
     * Ett enkelt tidur med start och stopp,
     *
     * Tiduret har en tidsgräns. Om den gränsen överskrids efter att tiduret
     * startas larmar tiduret.
     *
     * @property {TimerButton} button  Start/stopp-knapp.
     * @property {number}      elapsed Tid (ma) sedan tiduret startades.
     */
    class Timer {

        /**
         * Konstruera en instans av ett tidur.
         *
         * @param {number} duration Aktiveringstid i millisekunder.
         */
        constructor (duration) {
            this._button   = null;
            this._duration = duration;
            this._started  = 0;
            this._stopped  = 0;
            this._timer    = null;
        }

        // Åtkomstmetoder.
        get elapsed ()      {return this._stopped - this._started}
        set button (button) {this._button = button               }

        /**
         * Larma när aktiveringstiden löpt ut.
         *
         * @param {Timer} timer Den tidursinstans vars tid löpt ut.
         */
        expire (timer) {
            timer.stop();
            timer._button.expire();
        }

        /**
         * Starta tiduret.
         *
         * Returnerar true.
         *
         * @return {boolean}
         */
        start () {
            this._started = Date.now();
            this._stopped = 0;
            this._timer   = win.setTimeout(this.expire, this._duration, this);

            return true;
        }

        /**
         * Stoppa tiduret.
         *
         * Returnerar true om tiduret stoppats, eller false om det inte var
         * igång.
         *
         * @return {boolean}
         */
        stop () {
            if (null !== this._timer) {
                this._stopped = Date.now();
                win.clearTimeout(this._timer);
                this._timer = null;

                return true;
            }

            return false;
        }

    }


    /**
     * Knapp för att starta och stoppa ett tidur.
     */
    class TimerButton {

        /**
         * Konstruera en knappinstans.
         *
         * @param {HTMLElement} parent Föräldraelement i DOM-trädet.
         * @param {Timer}       timer  Tiduret som knappen ska styra.
         */
        constructor (parent, timer) {
            this._button = parent.insertBefore(
                doc.createElement("button"),
                parent.firstChild
            );
            this._timer  = timer;
            this._output = this._button.nextElementSibling;

            this._button.type      = "button";
            this._button.innerHTML = "Starta tidtagning";
            this._button.setAttribute("aria-pressed", "false");
            this._button.addEventListener("click", () => {
                this.stop() || this.start()
            });

            this._timer.button = this;
        }

        /**
         * Återställ knappen om tiduret löper ut.
         */
        expire () {
            this._output.classList.add("expired");
            this._output.innerHTML = `${this._timer.elapsed} ms`;
            this._button.innerHTML = "Starta tidtagning";
            this._button.setAttribute("aria-pressed", "false");
        }

        /**
         * Starta tiduret.
         *
         * Startar tiduren när användaren trycker på knappen medan den är
         * uppsläppt.
         *
         * Returnerar alltid true.
         *
         * @return {booelan}
         */
        start () {
            this._button.innerHTML = "Stoppa tidtagning";
            this._output.classList.remove("expired");
            this._output.innerHTML = "";
            this._button.setAttribute("aria-pressed", "true");
            this._timer.start();
            return true;
        }

        /**
         * Stoppa tiduret.
         *
         * Stoppar tiduren när användaren trycker på knappen igen medan den
         * är nedtryckt.
         *
         * Returnerar true om tiduret stoppats, eller false om knappen inte var
         * nedtryckt.
         *
         * @return {boolean}
         */
        stop () {
            if (this._timer.stop()) {
                this._output.classList.remove("expired");
                this._output.innerHTML = `${this._timer.elapsed} ms`;
                this._button.innerHTML = "Starta tidtagning";
                this._button.setAttribute("aria-pressed", "false");
                return true;
            }
            return false;
        }
    }


    /*
     * Initiiering.
     */

    // Skapa ett tidur och en styrknapp i varje element med class="timer".
    // Aktiveringstiden styrs av attributet data-duration i elementet.
    doc.querySelectorAll(".timer").forEach((container) => {
        const timer = new Timer(parseInt(container.dataset.duration, 10));
        const btn = new TimerButton(container, timer);
    });
})(window, document);
